/*
 * Needs to link against: Xlib.
 */

#include <stdio.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <sys/types.h>
#include <sys/time.h>
#include <signal.h>

Display *display;

int     mscreen;               /* number of displays in use */
char    *prog;                 /* the name of this program */

int debug;

#include "xev.h"

unsigned short AsciiToState(c)
    unsigned char c;
{
    return states[c];
}

KeyCode AsciiToKeycode(c)
    unsigned char c;
{
    return keycodes[c];
}

int escape_char = '~';


fixtty() {
    system("stty -cbreak -raw");

}

breaktty() {
    system("stty cbreak raw");
}

int main (int argc, char *argv[]) {
    KeySym keysym;
    Window nailedwindow;
    int done = 0;
    int lastc = 0;
    
    
    prog = *argv;
    
    if (argc == 1) {
	fprintf(stderr,
		"usage: %s nail WINDOWID {local | <display>} escape ##\n",
		prog);
	return 1;
    }
    
    /*
     * each argument is a display name except the magic "local" which 
     * will get transformed into the local display. 
     */
    
    while (*++argv != (char *) NULL) {
	if (!strcmp (*argv, "-debug")) {
	    debug++;
	    continue;
	}
	if(strcmp(*argv, "local") == 0) { 
	    *argv = "";
	}
	if(strcmp(*argv, "nail") == 0) {
	    argv++;
	    nailedwindow = atoi(*argv);
	    printf("nailed window: %d (0x%x)\n",nailedwindow,nailedwindow);
	    
	    continue;
	}
	if(strcmp(*argv, "escape") == 0) {
	    argv++;
	    escape_char = atoi(*argv);
	    printf("nailed window: %d (0x%x)\n",nailedwindow,nailedwindow);
	    
	    continue;
	}
	display = XOpenDisplay (*argv);
	if (display == (Display *) NULL) {
	    fprintf(stderr, "%s: unable to open display: %s\n", prog, *argv);
	    return 1;
	}
    }
    
#define STDIN 0
#define STDOUT 1
    signal(SIGCONT, breaktty);
    
    breaktty();
            
    while (1) {
	int no_state;
	
	/* deal with keyboard input */
	XEvent xev;
	XKeyEvent *xke = &xev.xkey;
	int xkstat;
	unsigned char c;

	read(STDIN, &c, 1);
				
	
	if(lastc == escape_char) {
	    lastc = 0;
	    if(c == escape_char) {
		/* just sending an escape... handled below */
	    } else switch(c) {
	    case 'Z'-'@':
		kill(getpid(), SIGTSTP);
		continue;
	    case '.':
		fixtty();
		exit(0);
	    default:
		{
		    char beep = 7;
		    write(STDOUT,&beep,1);
		    break;
		}
	    }			
	} else if(c == escape_char) {
	    lastc = c;
	    continue;
	}	
	    
	lastc = 0;
	
	if(debug) printf("Sending Key <%02x>\n",c);
	xke->type = KeyPress;
	xke->display = display;
	xke->window = nailedwindow;
	xke->state = AsciiToState(c);
	xke->keycode =AsciiToKeycode(c);

		    
	/* ??? what others are needed? */
	xkstat = XSendEvent(display, nailedwindow, True,
			    (ButtonPressMask | ButtonReleaseMask | 
			     KeyPressMask | KeyReleaseMask), &xev);
	if(xkstat == 0) {
	  fprintf(stderr, "xplay: Send KeyPress failed!\n");
	}
	xke->type = KeyRelease;
	xkstat = XSendEvent(display, nailedwindow, True,
			    (ButtonPressMask | ButtonReleaseMask | 
			     KeyPressMask | KeyReleaseMask), &xev);
	if(xkstat == 0) {
	  fprintf(stderr, "xplay: Send KeyRelease failed!\n");
	}
	
	XSync(display, False);
    }
}

